@@ -11,10 +11,24 @@ This software is released under an MIT/X11 open source license.
 Copyright 2012 Diffeo, Inc.
 '''
 
+import logging
+logger = logging.getLogger('streamcorpus')
+
 ## import the thrift library
 from thrift import Thrift
 from thrift.transport import TTransport
-from thrift.protocol import TBinaryProtocol
+from thrift.protocol.TBinaryProtocol import TBinaryProtocol, TBinaryProtocolAccelerated
+fastbinary_import_failure = None
+try:
+    from thrift.protocol import fastbinary
+    ## use faster C program to read/write
+    protocol = TBinaryProtocolAccelerated
+
+except Exception, exc:
+    fastbinary_import_failure = exc
+    ## fall back to pure python
+    protocol = TBinaryProtocol
+
 
 import os
 import uuid
@@ -33,7 +47,7 @@ def serialize(msg):
     Generate a serialized binary blob for a single message
     '''
     o_transport = StringIO()
-    o_protocol = TBinaryProtocol.TBinaryProtocol(o_transport)
+    o_protocol = protocol(o_transport)
     msg.write(o_protocol)
     o_transport.seek(0)
     return o_transport.getvalue()
@@ -76,6 +90,26 @@ class md5_file(object):
         self._md5.update(data)
         self._fh.write(data, *args, **kwargs)
 
+
+    def readAll(self, sz):
+        '''
+        This method allows TBinaryProtocolAccelerated to actually function.
+        Copied from here
+        http://svn.apache.org/repos/asf/hive/trunk/service/lib/py/thrift/transport/TTransport.py
+        '''
+        buff = ''
+        have = 0
+        while (have < sz):
+            chunk = self.read(sz-have)
+            have += len(chunk)
+            buff += chunk
+
+            if len(chunk) == 0:
+                raise EOFError()
+
+        return buff
+
+
     @property
     def md5_hexdigest(self):
         return self._md5.hexdigest()
@@ -110,6 +144,13 @@ class Chunk(object):
         :param message: defaults to StreamItem; you can specify your
         own Thrift-generated class here.
         '''
+        if not fastbinary_import_failure:
+            logger.debug('using TBinaryProtocolAccelerated (fastbinary)')
+
+        else:
+            logger.warn('import fastbinary failed; falling back to 15x slower TBinaryProtocol: %r'\
+                            % fastbinary_import_failure)
+
         allowed_modes = ['wb', 'ab', 'rb']
         assert mode in allowed_modes, 'mode=%r not in %r' % (mode, allowed_modes)
         self.mode = mode
@@ -203,7 +244,7 @@ class Chunk(object):
         if mode in ['ab', 'wb']:
             self._o_chunk_fh = md5_file( file_obj )
             self._o_transport = TTransport.TBufferedTransport(self._o_chunk_fh)
-            self._o_protocol = TBinaryProtocol.TBinaryProtocol(self._o_transport)
+            self._o_protocol = protocol(self._o_transport)
 
         else:
             assert mode == 'rb', mode
@@ -277,7 +318,7 @@ class Chunk(object):
         ## wrap the file handle in buffered transport
         i_transport = TTransport.TBufferedTransport(self._i_chunk_fh)
         ## use the Thrift Binary Protocol
-        i_protocol = TBinaryProtocol.TBinaryProtocol(i_transport)
+        i_protocol = protocol(i_transport)
 
         ## read message instances until input buffer is exhausted
         while 1:
